$ifndef _RTCC_24_INC_
$define _RTCC_24_INC_
'
' Real Time Clock Calendar peripheral procedures for PIC24H devices.
'
' This source code is provided 'as is' and any express or implied warranties, including,
' but not limited to, the implied warranties of merchantability and fitness for a particular
' purpose are disclaimed.
'
' In no event shall the author, Crownhill or contributers be liable for any direct, indirect, incidental,
' special, exemplary, or consequential damages (including, but not limited to,  procurement of substitute
' goods or services; loss of use, data, or profits;  or business interruption) however caused and on any
' theory of liability, whether in contract, strict liability, or tort (including negligence or otherwise)
' arising in any way out of the use of this source code, even if advised of the possibility of such damage.
'
' Any views and conclusions contained in the source code and any documentation are those of the author's
' and should not be interpreted as representing official policies,
' either expressed or implied, of Crownhill Associates Ltd.
'
' For use with the Crownhill Proton24 compiler
' Written by L.Johnson 17-02-14
'
'-------------------------------------------------------------------------------------
' Helper macros to point to a particular register
' Input     : None
' Output    : Pointer bits of RCFGCAL are manipulated
' Notes     : RTCPTR<1:0>: RTCC Value Register Window Pointer bits
'           : Points to the corresponding RTCC Value registers when reading RTCVALH and RTCVALL registers;
'           : the RTCPTR<1:0> value decrements on every read or write of RTCVALH until it reaches 00.
'           : RTCVAL.Byte1:
'           :   11 = Reserved
'           :   10 = Month
'           :   01 = Weekday
'           :   00 = Minutes
'           : RTCVAL.Byte0:
'           :   11 = Year
'           :   10 = Day
'           :   01 = Hours
'           :   00 = Seconds
'
$define RTCC_PointToYear() '
    Set RCFGCALbits_RTCPTR1'
    Set RCFGCALbits_RTCPTR0

$define RTCC_PointToMonth() '
    Set RCFGCALbits_RTCPTR1 '
    Clear RCFGCALbits_RTCPTR0

$define RTCC_PointToDay()   '
    Set RCFGCALbits_RTCPTR1 '
    Clear RCFGCALbits_RTCPTR0

$define RTCC_PointToWeekDay() '
    Clear RCFGCALbits_RTCPTR1 '
    Set RCFGCALbits_RTCPTR0

$define RTCC_PointToHours()   '
    Clear RCFGCALbits_RTCPTR1 '
    Set RCFGCALbits_RTCPTR0

$define RTCC_PointToMinutes() '
    Clear RCFGCALbits_RTCPTR1 '
    Clear RCFGCALbits_RTCPTR0

$define RTCC_PointToSeconds() '
    Clear RCFGCALbits_RTCPTR1 '
    Clear RCFGCALbits_RTCPTR0

'-------------------------------------------------------------------------------------
$define RTCC_WriteYear(pValue) RTCVAL.Byte0 = pValue
$define RTCC_WriteMonth(pValue) RTCVAL.Byte1 = pValue
$define RTCC_WriteDay(pValue) RTCVAL.Byte0 = pValue
$define RTCC_WriteWeekDay(pValue) RTCVAL.Byte1 = pValue
$define RTCC_WriteHours(pValue) RTCVAL.Byte0 = pValue
$define RTCC_WriteMinutes(pValue) RTCVAL.Byte1 = pValue
$define RTCC_WriteSeconds(pValue) RTCVAL.Byte0 = pValue

'-------------------------------------------------------------------------------------
$define RTCC_ReadYear() RTCVAL.Byte0
$define RTCC_ReadMonth() RTCVAL.Byte1
$define RTCC_ReadDay() RTCVAL.Byte0
$define RTCC_ReadWeekDay() RTCVAL.Byte1
$define RTCC_ReadHours() RTCVAL.Byte0
$define RTCC_ReadMinutes() RTCVAL.Byte1
$define RTCC_ReadSeconds() RTCVAL.Byte0

'-------------------------------------------------------------------------------------
' Convert a packed BCD (Binary Coded Decimal) value to a decimal value
' Input     : pValue holds the BCD value to convert to decimal
' Output    : Returns the decimal value
' Notes     : None
'
Proc RTCC_BCDToDec(pValue As Byte), Byte
    Result = (pValue >> 4) * 10
    Result = Result + (pValue & $0F)
EndProc

'-------------------------------------------------------------------------------------
' Convert a decimal value to packed BCD (Binary Coded Decimal) value
' Input     : pValue holds the decimal value to convert to BCD
' Output    : Returns the packed BCD value
' Notes     : None
'
Proc RTCC_DecToBCD(pValue As Byte), Byte
    Result = ((pValue / 10) << 4)
    Result = Result | (pValue // 10)
EndProc

'-------------------------------------------------------------------------------------
' Write the time to the RTCC
' Input     : pHour holds the hours value
'           : pMinute holds the minutes value
'           : pSecond holds the seconds value
' Output    : None
' Notes     : Parameter values are in decimal
'
Proc RTCC_SetTime(pHour As Byte, pMinute As Byte, pSecond As Byte)

    Dim bSaveWeekday As Byte = RTCC_GetWeekDay()    ' Preserve the weekdays value

    RTCC_PointToHours()                             ' Point to Hours
    RTCC_WriteHours(RTCC_DecToBCD(pHour))           ' Load Hours
    RTCC_WriteWeekDay(bSaveWeekday)                 ' Load the weekdays
    RTCC_WriteSeconds(RTCC_DecToBCD(pSecond))       ' Load the seconds
    RTCC_WriteMinutes(RTCC_DecToBCD(pMinute))       ' Load the minutes
EndProc

'-------------------------------------------------------------------------------------
' Write the date to the RTCC
' Input     : pDay holds the day value
'           : pMonth holds the months value
'           : pYear holds the year value
' Output    : None
' Notes     : Parameter values are in decimal
'
Proc RTCC_SetDate(pDay As Byte, pMonth As Byte, pYear As Byte)
    RTCC_PointToYear()                              ' Point to Year
    RTCC_WriteYear(RTCC_DecToBCD(pYear))            ' Load year

    RTCC_PointToDay()                               ' Point to Day
    RTCC_WriteDay(RTCC_DecToBCD(pDay))              ' Load day
    RTCC_WriteMonth(RTCC_DecToBCD(pMonth))          ' Load month
EndProc

'-------------------------------------------------------------------------------------
' Read the time from the RTCC
' Input     : None
' Output    : Indirectly loads the parameter variables with the Hours, Minutes and Seconds values
' Notes     : The returns are all in decimal
'
Proc RTCC_GetTime(ByRef pHour As Word, ByRef pMinute As Word, ByRef pSecond As Word)
    RTCC_PointToHours()                             ' Point to Hours
    Ptr8(pHour)   = RTCC_BCDToDec(RTCC_ReadHours())
    Ptr8(pMinute) = RTCC_BCDToDec(RTCC_ReadMinutes())
    Ptr8(pSecond) = RTCC_BCDToDec(RTCC_ReadSeconds())
EndProc

'-------------------------------------------------------------------------------------
' Read the date from the RTCC
' Input     : None
' Output    : Indirectly loads the parameter variables with the Year, Month and Day values
' Notes     : The returns are all in decimal
'
Proc RTCC_GetDate(ByRef pDay As Word, ByRef pMonth As Word, ByRef pYear As Word)
    RTCC_PointToYear()                              ' Point to Year
    Ptr8(pYear) = RTCC_BCDToDec(RTCC_ReadYear())    ' Get the year value

    RTCC_PointToMonth()                             ' Point to Month
    Ptr8(pMonth) = RTCC_BCDToDec(RTCC_ReadMonth())  ' Get the months value

    RTCC_PointToDay()                               ' Point to Day
    Ptr8(pDay) = RTCC_BCDToDec(RTCC_ReadDay())      ' Get the days value
EndProc

'-------------------------------------------------------------------------------------
' Get the weekday value from the RTCC
' Input     : None
' Output    : Returns the weekday value (0 - 6)
' Notes     : None
'
Proc RTCC_GetWeekDay(), Byte
    RTCC_PointToWeekDay()                   ' Point to Weekday (RTCVAL.Byte1)
    Result = RTCC_ReadWeekDay() & $07       ' Read the weekday value
EndProc

'-------------------------------------------------------------------------------------
' Write the weekday value to the RTCC
' Input     : pWeekDay holds the weekday value (0 - 6)
' Output    : None
' Notes     : None
'
Proc RTCC_SetWeekDay(pWeekDay As Byte)
    RTCC_PointToWeekDay()                   ' Point to Weekday (RTCVAL.Byte1)
    RTCC_WriteWeekDay(pWeekDay & $07)       ' Write the weekday value
EndProc

'-------------------------------------------------------------------------------------
' Return the texts following a particular date
' Input     : pDate holds the day value 1 to 31
' Output    : Returns a string holding the date value followed by a particular text. i.e. "st", "rd", "nd" or "th"
' Notes     : None
'
Proc RTCC_GetDateName(pDate As Byte), String * 5
    Push __OSM                                  ' Required because Str$ alters the stream value
    Result = Str$(Dec, pDate)
    Select pDate
        Case 11, 12, 13
            Result = Result + "th"
        Case Else
            pDate = RTCC_DecToBCD(pDate)
            If (pDate & $0F) = 3 Then
                Result = Result + "rd"
            ElseIf (pDate & $0F) = 2 Then
                Result = Result + "nd"
            ElseIf (pDate & $0F) = 1 Then
                Result = Result + "st"
            Else
                Result = Result + "th"
            EndIf
    EndSelect
    Pop __OSM
EndProc
'-----------------------------------------------------------------------------------------------------------------------------
' Return the name of the day
' Input     : pDayOfWeek holds the day value 0 to 6 (Sunday to Saturday)
' Output    : Returns a string holding the day's name
' Notes     : None
'
Proc RTCC_GetDayName(pDayOfWeek As Byte), String * 9
    Select pDayOfWeek
        Case 0
            Result = "Sunday"
        Case 1
            Result = "Monday"
        Case 2
            Result = "Tuesday"
        Case 3
            Result = "Wednesday"
        Case 4
            Result = "Thursday"
        Case 5
            Result = "Friday"
        Case 6
            Result = "Saturday"
        Case Else
            Clear Result
    EndSelect
EndProc

'-----------------------------------------------------------------------------------------------------------------------------
' Return the name of the month
' Input     : pMonth holds the month value 1 to 12 (January to December)
' Output    : Returns a string holding the month's name
' Notes     : None
'
Proc RTCC_GetMonthName(pMonth As Byte), String * 9
    Select pMonth
        Case 1
            Result = "January"
        Case 2
            Result = "February"
        Case 3
            Result = "March"
        Case 4
            Result = "April"
        Case 5
            Result = "May"
        Case 6
            Result = "June"
        Case 7
            Result = "July"
        Case 8
            Result = "August"
        Case 9
            Result = "September"
        Case 10
            Result = "October"
        Case 11
            Result = "November"
        Case 12
            Result = "December"
        Case Else
            Clear Result
    EndSelect
EndProc
'-------------------------------------------------------------------------------------
' Initialise the RTCC peripheral
' Input     : None
' Output    : None
' Notes     : None
'
Proc RTCC_Init()
'
' Unlock the OSCCON register and enable the secondary oscillator
'
    Disi #7                         ' Disable all interrupts for 7 cycles
    WREG1 = AddressOf(OSCCON.Byte0) ' Get the address of OSCCON.Byte0
    WREG0 = $02                     ' The value to place into OSCCON.Byte0
    WREG2 = $46                     ' \ Unlock sequence for OSCCON.Byte0
    WREG3 = $57                     ' /
    Ptr8(WREG1) = WREG2             ' \
    Ptr8(WREG1) = WREG3             ' | Indirectly load OSCCON.Byte0 with the above sequence
    Ptr8(WREG1) = WREG0             ' /
'
' Unlock the RCFGCAL register and enable the RTCC
'
    Disi #6                         ' Disable all interrupts for 6 cycles
    NVMKEY = $55                    ' \
    NVMKEY = $AA                    ' | Unlock RCFGCAL
    Set RCFGCALbits_RTCWREN         ' /
    Set RCFGCALbits_RTCEN           ' Enable the RTCC
EndProc

'-------------------------------------------------------------------------------------
_RTCC_Main:

$endif  ' _RTCC_24_INC_

